#include "StdAfx.h"
#include "DotLineDraw.h"

namespace {

//   ,   DotLineDraw()

struct DotLineDraw_t {
	HDC hdc;
	int phase;
	int cur_phase;
	int space[2];
	int start_x;
	int start_y;
	BOOL LastDotPainted;
};

void CALLBACK DotLineDrawProc(
  int X,          // x-coordinate of point
  int Y,          // y-coordinate of point
  LPARAM lpData   // application-defined data
  ) 
{
	DotLineDraw_t*	ld=(DotLineDraw_t*)lpData;

	if ((ld->start_x-X)*(ld->start_x-X)+(ld->start_y-Y)*(ld->start_y-Y)<ld->space[ld->cur_phase]*ld->space[ld->cur_phase]) {
		ld->LastDotPainted=FALSE;
		return;
	};
	
	if (ld->cur_phase==ld->phase) {
		MoveToEx(ld->hdc,ld->start_x,ld->start_y,NULL);
		LineTo(ld->hdc,X,Y);
	};

	ld->LastDotPainted=TRUE;
	ld->start_x=X;
	ld->start_y=Y;
	if (ld->cur_phase==1) {
		ld->cur_phase=0;
	} else {
		ld->cur_phase=1;
	};
};

}; //namespace

/* 
	 void DotLineDraw(HDC hdc,int x1,int y1,int x2,int y2,int width,
							int space1,COLORREF color1,int space2,COLORREF color2,
							BOOL OptimizeSpedForOddDots=FALSE)
	    .
	hdc -  
	x1,y1 -  
	x2,y2 -  
	width -  
	space1,color1 -      (space1<0 -   )
	space2,color2 -      (space2<0 -   )
	OptimizeSpedForOddDots -  TRUE,     
		(  " " + " "  "  " + " ")
		  -    
		  width        ;    
		 OptimizeSpedForOddDots=TRUE


: 

	  Windows9x  NT/2000

	      MM_TEXT (   )

	width  spaceX   , ..    width=30,  space=1 (width=30, space=10 - OK)

	    width>1     GDI  BeginPath() / EndPath() ( AbortPath())

	() 2001   al@twt.mpei.ac.ru

*/

void DotLineDraw(HDC hdc,int x1,int y1,int x2,int y2,int width,
			 int space1,COLORREF color1,int space2,COLORREF color2,
			 BOOL OptimizeSpedForOddDots)
{
	DotLineDraw_t ld;
	ld.hdc=hdc;
	ld.space[0]=space1;
	if (space1<0) {
		ld.space[0]*=-1;
	};
	ld.space[1]=space2;
	if (space2<0) {
		ld.space[1]*=-1;
	};

	if (width>1) {
		AbortPath(hdc);
	};

	LOGBRUSH logbrush;
	logbrush.lbStyle=BS_SOLID;
	logbrush.lbHatch=0;

	HPEN hPen;
	HPEN hOldPen;

	if (space1>0) {

		if (width>1) {
			logbrush.lbColor=color1;
			hPen=ExtCreatePen(PS_SOLID|PS_GEOMETRIC|PS_ENDCAP_FLAT,width,&logbrush,0,NULL);
		} else {
			hPen=CreatePen(PS_SOLID,0,color1);
		};
		hOldPen=(HPEN)SelectObject(hdc,hPen);

		if (width>1) {
			BeginPath(hdc);
		};

		if (OptimizeSpedForOddDots && space2>0) {
			MoveToEx(hdc,x1,y1,NULL);
			LineTo(hdc,x2,y2);
		} else {
			ld.start_x=x1;
			ld.start_y=y1;
			ld.cur_phase=0;
			ld.phase=0;
			ld.LastDotPainted=TRUE;
			LineDDA(x1,y1,x2,y2,DotLineDrawProc,(LPARAM)&ld);
			if (!ld.LastDotPainted && ld.cur_phase==0) {
				MoveToEx(hdc,ld.start_x,ld.start_y,NULL);
				LineTo(hdc,x2,y2);
			};
		};
		if (width>1) {
			EndPath(hdc);
			StrokePath(hdc);
			AbortPath(hdc);
		};

		SelectObject(hdc,hOldPen);
		DeleteObject(hPen);
	};
	if (space2>0) {
		if (width>1) {
			logbrush.lbColor=color2;
			hPen=ExtCreatePen(PS_SOLID|PS_GEOMETRIC|PS_ENDCAP_FLAT,width,&logbrush,0,NULL);
		} else {
			hPen=CreatePen(PS_SOLID,0,color2);
		};
		hOldPen=(HPEN)SelectObject(hdc,hPen);

		if (width>1) {
			BeginPath(hdc);
		};

		ld.start_x=x1;
		ld.start_y=y1;
		ld.cur_phase=0;
		ld.phase=1;
		ld.LastDotPainted=TRUE;
		LineDDA(x1,y1,x2,y2,DotLineDrawProc,(LPARAM)&ld);
		if (!ld.LastDotPainted && ld.cur_phase==1) {
			MoveToEx(hdc,ld.start_x,ld.start_y,NULL);
			LineTo(hdc,x2,y2);
		};

		if (width>1) {
			EndPath(hdc);
			StrokePath(hdc);
			AbortPath(hdc);
		};

		SelectObject(hdc,hOldPen);
		DeleteObject(hPen);
	};
};

